#
# SPEC-fishing.R,  9 Mar 25
#
# Data from:
# www.spec.org

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(2)

# Extract details of memory nterface used
DRAM_info=function(mem_str)
{
mem_str=toupper(mem_str)
PC_pos=regexpr("PC[2-5]?L?[ -]+[0-9]+[A-Z]?[-,]?", mem_str)
PC_str=substr(mem_str, PC_pos, PC_pos+attr(PC_pos, "match.length")-1)

DDR_pos=regexpr("DDR[2-5]?[ -\\)][0-9]*", mem_str)
DDR_str=substr(mem_str, DDR_pos, DDR_pos+attr(DDR_pos, "match.length")-1)

ddr_missing=(DDR_str == "")
DDR_str[ddr_missing]=PC_str[ddr_missing]

DDR_str=sub("[ ,-\\)]?$", "", DDR_str)
DDR_str=sub(" - ", "-", DDR_str)
DDR_str=sub(" ", "-", DDR_str)
DDR_str=sub("[A-Z]$", "", DDR_str)

# Fix various 'typos' in reported names
DDR_str=sub("DDR40$", "DDR-400", DDR_str)
DDR_str=sub("DDR-40$", "DDR-400", DDR_str)
DDR_str=sub("DDR2-40$", "DDR2-400", DDR_str)
DDR_str=sub("DDR2-53$", "DDR2-530", DDR_str)
DDR_str=sub("DDR2-530$", "DDR2-533", DDR_str)
DDR_str=sub("DDR2-66$", "DDR2-667", DDR_str)
DDR_str=sub("DDR2-80$", "DDR2-800", DDR_str)
DDR_str=sub("DDR3-106$", "DDR3-1066", DDR_str)
DDR_str=sub("DDR3-1060$", "DDR3-1066", DDR_str)
DDR_str=sub("DDR3-13$", "DDR3-1333", DDR_str)
DDR_str=sub("DDR3-133$", "DDR3-1333", DDR_str)
DDR_str=sub("DDR3-160$", "DDR3-1600", DDR_str)
DDR_str=sub("DDR4-160$", "DDR4-1600", DDR_str)
DDR_str=sub("PC2-320$", "PC2-3200", DDR_str)
DDR_str=sub("PC2-420$", "PC2-4200", DDR_str)
DDR_str=sub("PC2-530$", "PC2-5300", DDR_str)
DDR_str=sub("PC2-640$", "PC2-6400", DDR_str)
DDR_str=sub("PC3-16000$", "PC3-1600", DDR_str)
DDR_str=sub("PC3L-128000$", "PC3L-12800", DDR_str)
DDR_str=sub("PC3L-", "PC3-", DDR_str) # L is low voltage
DDR_str=sub("PC4-2667$", "PC4-2666", DDR_str) # 2667 is technically correct, but not used often

# How many non-matches
# length(c06$Memory[which(DDR_str == PC_str)])
# table(c06$Memory[which(DDR_str == PC_str)])

return(DDR_str)
}


c06=read.csv("cpu2006-results-20250303.csv")
c17=read.csv("cpu2017-results-20250303.csv")
ddr_pc=read.csv("DDR_PC.csv")
ddr_info=read.csv("DDR_Memory_specs.csv")


c06$Date=as.Date(paste0("15-", c06$Test.Date), format="%d-%b-%Y")
c17$Date=as.Date(paste0("15-", c17$Test.Date), format="%d-%b-%Y")

d_2005=as.Date("01-01-2005", format="%d-%m-%Y")
d_2017=as.Date("15-01-2017", format="%d-%m-%Y")
d_2020=as.Date("15-01-2020", format="%d-%m-%Y")
d_2025=as.Date("15-01-2025", format="%d-%m-%Y")

x_dates=c(d_2005, d_2017, d_2020, d_2025)


# Which systems appear in both 2006 and 2017 versions of SPEC results?
cint06=subset(c06, Benchmark == "CINT2006")
cint17=subset(c17, Benchmark == "CINT2017")

cint_all=merge(cint06, cint17, by="System")

# t=which (cint_all$Processor.MHz.x != cint_all$Processor.MHz.y)
# cint_all[t, c("Processor.MHz.x", "Processor.MHz.y")]
#     Processor.MHz.x Processor.MHz.y
# 243            1700            1800
# 401            2000            3000
# 444            3000            3200
# 446            2600            3400
cint_all=subset(cint_all, Processor.MHz.x == Processor.MHz.y)

plot(cint_all$Baseline, cint_all$Base.Result, log="xy", col=pal_col[2],
	xlab="SPEC 2006", ylab="SPEC 2017")

# cint_mod=glm(log(Base.Result) ~ log(Baseline), data=cint_all)
# cint_mod=glm(log(Base.Result) ~ log(Baseline)+I(log(Baseline)^2), data=cint_all,
cint_mod=glm(log(Base.Result) ~ log(Baseline), data=cint_all,
						subset=(Baseline > 30))
summary(cint_mod)

x_base=seq(30, 90, by=10)
pred=exp(predict(cint_mod, newdata=data.frame(Baseline=x_base)))

x_base/pred

lines(x_base, pred, col=pal_col[1])


# Extract processor family name
get_proc_name=function(p_str, vendor, ven_regex)
{
base=regexpr(vendor, p_str)
if (base == -1)
   return("")

m_len=aaply(ven_regex, .mar=1, function(X) {m=regexpr(X, p_str); attr(m, "match.length")})
max_len=max(c(nchar(vendor), m_len))

return(substr(p_str, base, base+max_len-1))
}


amd=read.csv("AMD_Processors.csv")
intel_regex=read.csv("Intel_Processors.csv")

amd_regex=paste0("AMD", "[ ]*", c(unique(amd$Familyi), "A10", "A[4-9]", "FX", "EPYC", "Ryzen"))
amd_names=aaply(c17$Processor, .mar=1, get_proc_name, "AMD", amd_regex)
# table(amd_names)

intel_names=aaply(c17$Processor, .mar=1, get_proc_name, "Intel", intel_regex)
# table(Intel_names)

c17$amd_names=amd_names
c17$inteld_names=intel_names

c17$proc_names=amd_names
no_names=(amd_names == "")
c17$proc_names[no_names]=intel_names[no_names]

# c17$Processor[which(c17$proc_names == "")]


# What DDR/PC memory is installed?
c17$DRAM=DRAM_info(c17$Memory)

# Fit model to 2017 version of INT benchmark
# To fit 2006 version, change c17 -> c06
c17_int=subset(c17, proc_names != "")
c17_int=subset(c17_int, Benchmark == "CINT2017") # Integer benchmark results

plot(c17_int$Processor.MHz, c17_int$Result, log="xy",
	xlim=c(1000, 5000),
	xlab="Clock rate", ylab="cpu2017 INT Result")

# Fit a model using memory names
c17_mod=glm(log(Result) ~ log(Processor.MHz) + DRAM+proc_names, data=c17_int,
						subset=(Result > 0))
summary(c17_mod)

# Plot integer Result against date and fit r simple egression model
plot(c17_int$Date, c17_int$Result, col=pal_col[2],
	ylim=c(4, 22),
	xlab="Date", ylab="cpu2017 INT Result")

res_mod=glm(Result ~ Date, data=c17_int, subset=(Result > 3))
summary(res_mod)

pred=predict(res_mod, newdata=data.frame(Date=x_dates))
lines(x_dates, pred, col=pal_col[1])

dev.copy(dev=png, file="c2017INT-res.png")
dev.off()


# Map DRAM names to DDR version + Bandwidth
c17$DDR=mapvalues(c17$DRAM, ddr_pc$PC, ddr_pc$DDR)

c17$mem_bandwidth=as.numeric(mapvalues(c17$DDR, ddr_info$Standard, ddr_info$Bandwidth_MB.s))
c17_int=subset(c17, proc_names != "") 
c17_int=subset(c17_int, Benchmark == "CINT2017")

# Fit a multiplicative model using memory bandwidth
# mem_mod=glm(log(Result) ~ log(Processor.MHz) -1 + log(mem_bandwidth)*proc_names, data=c17_int,
# mem_mod=glm(log(Result) ~ -1+log(Processor.MHz) * log(mem_bandwidth)+proc_names, data=c17_int,
mem_mod=glm(log(Result) ~ log(Processor.MHz) -1 + log(mem_bandwidth)+proc_names, data=c17_int,
						subset=(Result > 0))
summary(mem_mod)

# Memory bandwidth against date
plot(c17_int$Date, c17_int$mem_bandwidth, col=pal_col[2],
	# xaxs="i",
	# xlim=c(d_2005, d_2025),
	xlab="Date", ylab="Memory Bandwidth (MB/sec\n)")

date_mod=glm(mem_bandwidth ~ Date, data=c17_int)
summary(date_mod)

pred=predict(date_mod, newdata=data.frame(Date=x_dates))
lines(x_dates, pred, col=pal_col[1])


plot(c17_int$mem_bandwidth, c17_int$Result, log="y",
	xlab="Memory bandwidth (MB/sec)", ylab="cpu2017 INT Result")

memres_mod=glm(Result ~ mem_bandwidth, data=c17_int,
					subset=(Result > 3))
summary(memres_mod)

# Divide by 1024 to remove the clutter on the y-axis
plot(c17_int$Processor.MHz/1024, c17_int$mem_bandwidth/1024, col=pal_col[2],
	xlab="CPU frequency (GHz)", ylab="Memory Bandwidth (GB/s)")

MHzmem_mod=glm(log(mem_bandwidth) ~ log(Processor.MHz)+I(log(Processor.MHz)^2)-1, data=c17_int)
summary(MHzmem_mod)

x_MHz=seq(2000, 4500, by=100)
pred=predict(MHzmem_mod, newdata=data.frame(Processor.MHz=x_MHz))

lines(x_MHz/1024, exp(pred)/1024, col=pal_col[1])

dev.copy(dev=png, file="SPEC17-cpu-memtran.png")
dev.off()

# range(c17_int$Processor.MHz)
# # [1] 1700 4700
# 47/17
# # [1] 2.764706
# (47/17)^0.2
# # [1] 1.225546
# range(c17_int$mem_bandwidth, na.rm=TRUE)
# # [1] 19200 51200
# 512/192
# # [1] 2.666667
# (512/192)^0.72
# # [1] 2.026271

# Range of processor family impact
#  range(exp(coef(mem_mod)[c(-1, -2)]))
# # [1] 0.0007444265 0.0016651218
#  1665/744
# # [1] 2.237903

# Are number of cores a predictor than processor family?
Cmem_mod=glm(log(Result) ~ log(Processor.MHz) -1 + log(mem_bandwidth)+X..Cores, data=c17_int,
						subset=(Result > 0))
summary(Cmem_mod)


# Has the numbe rof cores been increasing over time?
# Yes, slightly
plot(c17_int$Date, c17_int$X..Cores, log="y",
	 xlim=c(d_2017, d_2025), ylim=c(1, 500))

core_mod=glm(X..Cores ~ Date, data=c17_int)
summary(core_mod)

